/*
 * channel.h:	Get and forward AC3 stream from DVB card to the 
 *		S/P-DIF interface of sound cards supported by ALSA. 
 *		(NO decoding!)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * Or, point your browser to http://www.gnu.org/copyleft/gpl.html
 *
 * Copyright (C) 2002,2003 Werner Fink, <werner@suse.de>
 */

#ifndef __CHANNEL_H
#define __CHANNEL_H

#include <vdr/receiver.h>
#include <vdr/transfer.h>
#include <vdr/status.h>
#include <vdr/thread.h>
#include "spdif.h"
#include "bounce.h"
#include "bitstreamout.h"
#include "pts.h"

#include "mp2_decoder.h"

#ifndef TS_SIZE
# define TS_SIZE	188
#endif
#ifndef TS_ERROR
# define TS_ERROR	0x80
#endif
#ifndef PAY_START
# define PAY_START	0x40
#endif
#ifndef CONT_CNT_MASK
# define CONT_CNT_MASK	0x0F
#endif
#ifndef ADAPT_MASK
# define ADAPT_MASK	0x30
#endif
#ifndef ADAPT_FIELD
# define ADAPT_FIELD	0x20
#endif
#ifndef PTS_ONLY
# define PTS_ONLY	0x80
#endif

class cChannelOutSPDif;

class cInStream : public cReceiver, cThread {
private:
    volatile flags_t flags;
    // Sync/Underrun thread
    #define FLAG_RUNNING	0x01
    #define FLAG_ACTIVE		0x02
    #define FLAG_FAILED		0x04
    #define FLAG_BOUNDARY	0x08
    #define FLAG_PAYSTART	0x10
    // The TS scanner
    static const uint_32 PS1magic;
    // Stream detection
    iec60958* stream;
    uint_16 apid;
    bool    apidIsDD;
    uint_16 bfound;
    uint_16 paklen;
    uint_16 ptsoff;
    uchar   ptsflag;
    uint_32 syncword;
    inline void ScanTSforPS1(const uint8_t *buf, const int cnt);
    inline void ScanTSforMP2(const uint_8 *tsdata, int len);
    inline void HardSyncReset();
    // Fast ring buffer
    static uint_8  * tsdata;
    static cBounce * bounce;
    cPTS pts;
    cDevice *PrimaryDevice;
protected:
    spdif *const spdifDev;
    ctrl_t &setup;
    MP2Decoder MP2Dec;

    virtual void Action(void);
    virtual void Activate(bool on);
    virtual void Receive(uchar *b, int cnt);
public:
    cInStream( uint_16 aPid, bool aPidIsDD, spdif *dev, ctrl_t &up, cBounce *bPtr);
    ~cInStream();
    uint_16  getApid() { return apid; }
    bool     getApidIsDD() { return apidIsDD; }
};

class cChannelOutSPDif : public cStatus, cThread  {
private:
    volatile flags_t flags;
    #define FLAG_RUNNING	0x01
    #define FLAG_ACTIVE		0x02
    #define FLAG_CHANNELSET	0x04
    cInStream *in;
    bool replaying;
    const cControl *replayControl;
    const char *replayName;
    int ChannelNumber; 
    uint_16 Apid;
    uint_16 ApidOrig;
    bool ApidIsDD;
    bool ApidLocked;
    virtual void IfNeededMuteSPDIF(void);
    static cBounce * bounce;
protected:
    const char *const SPDIFmute;
    spdif *const spdifDev;
    ctrl_t &setup;
    virtual void ChannelSwitch(const cDevice *Device, int channelNumber);
    virtual void Replaying(const cControl *Control, const char *Name);
public:
    virtual void AudioSwitch();
    virtual void AudioSwitch(uint_16 apid, bool apidisdd);
    virtual void Activate(bool on);
    virtual void Action(void);
    void ActivateInStream(bool on);
    cChannelOutSPDif(spdif &dev, ctrl_t &up, cBounce * bPtr, const char *script);
    ~cChannelOutSPDif(void);
    uint_16  getApid();
    bool getApidIsDD();
    int getChannelNumber() { return ChannelNumber; } 
    bool GetCurrentAudioTrack(uint_16 &apid, bool &apidIsDD);
    bool GetAudioTrack(int num, bool dd, int &apid);
    static bool GetCurrentAudioTrack(uint_16 &apid, bool &apidIsDD, cChannel *channel);
};

#endif // __CHANNEL_H
